const Gtk = imports.gi.Gtk;
const Gio = imports.gi.Gio;
const Gdk = imports.gi.Gdk;
const Lang = imports.lang;

const Gettext = imports.gettext.domain('shamsi-calendar');
const _ = Gettext.gettext;

let extension = imports.misc.extensionUtils.getCurrentExtension();
let convenience = extension.imports.convenience;

let Schema = convenience.getSettings('org.gnome.shell.extensions.shamsi-calendar');

function init() {
}

const App = new Lang.Class({
    Name: 'ShamsiCalendar.App',

    _init: function () {
        this.main_hbox = new Gtk.Box({
            orientation: Gtk.Orientation.HORIZONTAL,
            spacing: 20,
            border_width: 10
        });
        this.vbox3 = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            spacing: 10,
            border_width: 10
        });

        this.vbox2 = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            spacing: 10,
            border_width: 10
        });

        this.vbox1 = new Gtk.Box({
            orientation: Gtk.Orientation.VERTICAL,
            spacing: 10,
            border_width: 10
        });

        this.main_hbox.pack_start(this.vbox3, false, false, 0);
        this.main_hbox.pack_start(this.vbox2, false, false, 0);
        this.main_hbox.pack_start(this.vbox1, false, false, 0);

        let item,label,format,hbox,comment;


        // COLOR
        this.vbox3.add(new Gtk.Label({ label: _('حالت نمایش در نوار وضعیت:') }));

        this.vbox3.add(new Gtk.Label({ label: _('موقعیت') }));
        item = new Gtk.ComboBoxText();
        item.append('left', 'سمت چپ');
        item.append('center', 'وسط');
        item.append('right', 'سمت راست');
        item.set_active(Schema.get_enum('position'));
        this.vbox3.add(item);
        Schema.bind('position', item, 'active-id', Gio.SettingsBindFlags.DEFAULT);

        item = new Gtk.CheckButton({ label: _('رنگ سفارشی متن') });
        this.vbox3.add(item);
        Schema.bind('custom-color', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        let color = new Gtk.ColorButton();
        this.vbox3.add(color);

        let _color = this.getColorByHexadecimal(Schema.get_string('color'));
        color.set_color(_color);

        color.connect('color-set', (function (innerColor) {
            Schema.set_string('color', this.getHexadecimalByColor(innerColor.get_color()));
        }).bind(this));

        item = new Gtk.CheckButton({ label: _('اعلان هنگام راه‌اندازی') });
        this.vbox3.add(item);
        Schema.bind('startup-notification', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        format = new Gtk.Entry();
        label = new Gtk.Label({ label: 'قالب تاریخ درون نوار وضعیت: ' });
        hbox = new Gtk.HBox();
        hbox.add(format);
        hbox.add(label);
        this.vbox3.add(hbox);
        format.set_text(Schema.get_string('widget-format'));
        format.connect('changed', function (innerFormat) {
            Schema.set_string('widget-format', innerFormat.text);
        });

        comment = new Gtk.Label({
            label: _('<span size="x-small">قالب‌های قابل استفاده:\n%Y: سال چهار رقمی\n%y: سال دو رقمی\n%M: ماه دو رقمی\n%m: ماه یک یا دو رقمی\n%MM: نام کامل ماه\n%mm: خلاصه نام ماه\n%D: روز دو رقمی\n%d: روز یک یا دو رقمی\n%WW: نام روز هفته\n%ww: خلاصه نام روز هفته</span>'),
            use_markup: true
        });
        this.vbox3.add(comment);

        // FONT
        /* item = new Gtk.CheckButton({label: _('Use custom font')})
         this.vbox3.add(item)
         Schema.bind('custom-font', item, 'active', Gio.SettingsBindFlags.DEFAULT);

         label = new Gtk.Label({label: "Font: "});
         let font = new Gtk.FontButton();
         font.set_show_size(false);
         //font.set_show_style(false);

         let _actor = new Gtk.HBox();
         _actor.add(label);
         _actor.add(font);
         font.set_font_name(Schema.get_string('font'));

         this.vbox3.add(_actor);
         font.connect('font-set', function(font){
         Schema.set_string('font', font.get_font_name());
         });*/



        // EVENTS
        this.vbox2.add(new Gtk.Label({
            label: _('تنظیمات نمایش مناسبت‌ها:\n<span size="x-small">(مناسبت‌های رسمی و غیر رسمی قابل نمایش)</span>'),
            use_markup: true
        }));

        item = new Gtk.CheckButton({ label: _('مناسبت‌های رسمی کشور: هجری‌قمری') });
        this.vbox2.add(item);
        Schema.bind('event-iran-solar', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        item = new Gtk.CheckButton({ label: _('مناسبت‌های رسمی کشور: هجری‌شمسی') });
        this.vbox2.add(item);
        Schema.bind('event-iran-lunar', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        item = new Gtk.CheckButton({ label: _('مناسبت‌های رسمی کشور: میلادی (جهانی)') });
        this.vbox2.add(item);
        Schema.bind('event-world', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        item = new Gtk.CheckButton({ label: _('مناسبت‌های غیر رسمی ایران باستان') });
        this.vbox2.add(item);
        Schema.bind('event-persian', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        /* Merged to iranSolar(event-iran-solar): */
        // item = new Gtk.CheckButton({label: _('Persian personages')});
        // this.vbox2.add(item);
        // Schema.bind('event-persian-personage', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        // DATES FORMAT
        this.vbox1.add(new Gtk.Label({ label: _('نمایش تاریخ‌ها:') }));

        item = new Gtk.CheckButton({ label: _('هجری شمسی') });
        this.vbox1.add(item);
        Schema.bind('persian-display', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        format = new Gtk.Entry();
        label = new Gtk.Label({ label: '     قالب: ' });
        hbox = new Gtk.HBox();
        hbox.add(format);
        hbox.add(label);
        this.vbox1.add(hbox);
        format.set_text(Schema.get_string('persian-display-format'));
        format.connect('changed', function (innerFormat) {
            Schema.set_string('persian-display-format', innerFormat.text);
        });

        item = new Gtk.CheckButton({ label: _('هجری قمری') });
        this.vbox1.add(item);
        Schema.bind('hijri-display', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        format = new Gtk.Entry();
        label = new Gtk.Label({ label: '     قالب: ' });
        hbox = new Gtk.HBox();
        hbox.add(format);
        hbox.add(label);
        this.vbox1.add(hbox);
        format.set_text(Schema.get_string('hijri-display-format'));
        format.connect('changed', function (innerFormat) {
            Schema.set_string('hijri-display-format', innerFormat.text);
        });

        item = new Gtk.CheckButton({ label: _('میلادی') });
        this.vbox1.add(item);
        Schema.bind('gregorian-display', item, 'active', Gio.SettingsBindFlags.DEFAULT);

        format = new Gtk.Entry();
        label = new Gtk.Label({ label: '     قالب: ' });
        hbox = new Gtk.HBox();
        hbox.add(format);
        hbox.add(label);
        this.vbox1.add(hbox);
        format.set_text(Schema.get_string('gregorian-display-format'));
        format.connect('changed', function (innerFormat) {
            Schema.set_string('gregorian-display-format', innerFormat.text);
        });

        comment = new Gtk.Label({
            label: _('<span size="x-small">قالب‌های قابل استفاده:\n%Y: سال چهار رقمی\n%y: سال دو رقمی\n%M: ماه دو رقمی\n%m: ماه یک یا دو رقمی\n%MM: نام کامل ماه\n%mm: خلاصه نام ماه\n%D: روز دو رقمی\n%d: روز یک یا دو رقمی\n%WW: نام روز هفته\n%ww: خلاصه نام روز هفته</span>'),
            use_markup: true
        });
        this.vbox1.add(comment);




        this.main_hbox.show_all();
    },

    _scaleRound: function (value) {
        // Based on gtk/gtkcoloreditor.c
        value = Math.floor((value / 255) + 0.5);
        value = Math.max(value, 0);
        value = Math.min(value, 255);
        return value;
    },

    _dec2Hex: function (value) {
        value = value.toString(16);

        while (value.length < 2) {
            value = '0' + value;
        }

        return value;
    },

    getColorByHexadecimal: function (hex) {
        let colorArray = Gdk.Color.parse(hex);
        let color = null;

        if (colorArray[0]) {
            color = colorArray[1];
        } else {
            // On any error, default to red
            color = new Gdk.Color({ red: 65535 });
        }

        return color;
    },

    getHexadecimalByColor: function (color) {
        let red = this._scaleRound(color.red);
        let green = this._scaleRound(color.green);
        let blue = this._scaleRound(color.blue);
        return '#' + this._dec2Hex(red) + this._dec2Hex(green) + this._dec2Hex(blue);
    }
});

function buildPrefsWidget() {
    let widget = new App();
    return widget.main_hbox;
}
